#!/usr/bin/env python
# -*- coding:utf-8 -*-

__author_name__ = 'Mu Yang'
__author_email__ = 'emfomy@gmail.com'
__copyright__ = '2018-2020 CKIP Lab'

__title__ = 'E-HowNet'
__version__ = '0.5.2'
__description__ = 'E-HowNet Utilities'
__license__ = "CC-BY-NC-SA 4.0"

__url__ = 'https://ehownet.readthedocs.io'
__download_url__ = __url__+'/tarball/'+__version__
